@@ -28,6 +28,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  */
+#include "tclInt.h"
 #include "itclInt.h"
 /*
  * ------------------------------------------------------------------------
@@ -680,7 +681,6 @@ Itcl_ScopeCmd(
     Tcl_HashEntry *hPtr;
     Tcl_Object oPtr;
     Tcl_InterpDeleteProc *procPtr;
-    Tcl_Obj *objPtr;
     Tcl_Obj *objPtr2;
     Tcl_Var var;
     Tcl_HashEntry *entry;
@@ -767,9 +767,15 @@ Itcl_ScopeCmd(
             Tcl_Obj *resultPtr = Tcl_GetObjResult(interp);
 	    if (vlookup->ivPtr->protection != ITCL_PUBLIC) {
 	        Tcl_AppendToObj(resultPtr, ITCL_VARIABLES_NAMESPACE, -1);
+		Tcl_AppendToObj(resultPtr, (Tcl_GetObjectNamespace(
+			contextIclsPtr->oPtr))->fullName, -1);
+		Tcl_AppendToObj(resultPtr, "::", -1);
+		Tcl_AppendToObj(resultPtr,
+			Tcl_GetString(vlookup->ivPtr->namePtr), -1);
+	    } else {
+		Tcl_AppendToObj(resultPtr,
+			Tcl_GetString(vlookup->ivPtr->fullNamePtr), -1);
 	    }
-            Tcl_AppendToObj(resultPtr,
-	        Tcl_GetString(vlookup->ivPtr->fullNamePtr), -1);
             if (openParen) {
                 *openParen = '(';
                 Tcl_AppendToObj(resultPtr, openParen, -1);
@@ -813,22 +819,13 @@ Itcl_ScopeCmd(
 	        doAppend = 0;
             }
         }
-        objPtr = Tcl_NewStringObj((char*)NULL, 0);
-        Tcl_IncrRefCount(objPtr);
-	if (doAppend) {
-	    Tcl_AppendToObj(objPtr, "::", -1);
-	    Tcl_AppendToObj(objPtr,
-	            Tcl_GetString(contextIoPtr->namePtr), -1);
-	} else {
-	    Tcl_AppendToObj(objPtr, "::", -1);
-	    Tcl_AppendToObj(objPtr,
-	            Tcl_GetString(contextIoPtr->namePtr), -1);
-	}
 
         objPtr2 = Tcl_NewStringObj((char*)NULL, 0);
         Tcl_IncrRefCount(objPtr2);
 	Tcl_AppendToObj(objPtr2, ITCL_VARIABLES_NAMESPACE, -1);
-	Tcl_AppendToObj(objPtr2, Tcl_GetString(objPtr), -1);
+	Tcl_AppendToObj(objPtr2,
+		(Tcl_GetObjectNamespace(contextIoPtr->oPtr))->fullName, -1);
+
         if (doAppend) {
             Tcl_AppendToObj(objPtr2,
 	            Tcl_GetString(vlookup->ivPtr->fullNamePtr), -1);
@@ -843,8 +840,8 @@ Itcl_ScopeCmd(
             Tcl_AppendToObj(objPtr2, openParen, -1);
             openParen = NULL;
         }
-        Tcl_AppendElement(interp, Tcl_GetString(objPtr2));
-        Tcl_DecrRefCount(objPtr);
+        /* fix for SF bug #238 use Tcl_AppendResult instead of Tcl_AppendElement */
+        Tcl_AppendResult(interp, Tcl_GetString(objPtr2), NULL);
         Tcl_DecrRefCount(objPtr2);
     } else {
 
@@ -1336,8 +1333,6 @@ Itcl_MixinAddCmd(
     Tcl_Obj *const objv[])   /* argument objects */
 {
     Tcl_Obj **newObjv;
-    ItclObjectInfo *infoPtr;
-    ItclClass *iclsPtr;
     int result;
 
     ItclShowArgs(1, "Itcl_MixinAddCmd", objc, objv);
@@ -1345,8 +1340,6 @@ Itcl_MixinAddCmd(
         Tcl_WrongNumArgs(interp, 1, objv, "<className> <mixinName> ?<mixinName> ...?");
         return TCL_ERROR;
     }
-    infoPtr = Tcl_GetAssocData(interp, ITCL_INTERP_DATA, NULL);
-    iclsPtr = (ItclClass*)Itcl_PeekStack(&infoPtr->clsStack);
     newObjv = (Tcl_Obj **)ckalloc(sizeof(Tcl_Obj *)*(objc+1));
     newObjv[0] = Tcl_NewStringObj("::oo::define", -1);
     Tcl_IncrRefCount(newObjv[0]);
@@ -1404,10 +1397,8 @@ Itcl_NWidgetCmd(
     Tcl_Obj *const objv[])   /* argument objects */
 {
     ItclClass *iclsPtr;
-    ItclObjectInfo *infoPtr;
     int result;
 
-    infoPtr = (ItclObjectInfo *)clientData;
     iclsPtr = NULL;
     ItclShowArgs(0, "Itcl_NWidgetCmd", objc-1, objv);
     result = ItclClassBaseCmd(clientData, interp, ITCL_ECLASS|ITCL_NWIDGET, objc, objv,
@@ -1750,20 +1741,16 @@ Itcl_AddComponentCmd(
     Tcl_CallFrame frame;
     Tcl_Var varPtr;
     ItclVarLookup *vlookup;
-    ItclObjectInfo *infoPtr;
     ItclObject *contextIoPtr;
     ItclClass *contextIclsPtr;
     ItclComponent *icPtr;
     ItclVariable *ivPtr;
     const char *varName;
-    const char *name;
     int isNew;
     int result;
-    int type = VAR_TYPE_VARIABLE;
 
     result = TCL_OK;
     contextIoPtr = NULL;
-    infoPtr = (ItclObjectInfo *)clientData;
     ItclShowArgs(1, "Itcl_AddComponentCmd", objc, objv);
     if (objc < 3) {
         Tcl_WrongNumArgs(interp, 1, objv, 
@@ -1788,19 +1775,17 @@ Itcl_AddComponentCmd(
 		Tcl_GetString(objv[1]), "\"", NULL);
         return TCL_ERROR;
     }
-    if (ItclCreateComponent(interp, contextIclsPtr, objv[2], 0,
+    if (ItclCreateComponent(interp, contextIclsPtr, objv[2], /* not common */0,
             &icPtr) != TCL_OK) {
         return TCL_ERROR;
     }
+    ItclAddClassComponentDictInfo(interp, contextIclsPtr, icPtr);
     contextIclsPtr->numVariables++;
     Tcl_SetHashValue(hPtr, icPtr);
     Tcl_DStringInit(&buffer);
     Tcl_DStringAppend(&buffer, ITCL_VARIABLES_NAMESPACE, -1);
-    name = Tcl_GetString(contextIoPtr->namePtr);
-    if ((name[0] != ':') && (name[1] != ':')) {
-            Tcl_DStringAppend(&buffer, "::", 2);
-    }
-    Tcl_DStringAppend(&buffer, name, -1);
+    Tcl_DStringAppend(&buffer,
+	    (Tcl_GetObjectNamespace(contextIoPtr->oPtr))->fullName, -1);
     Tcl_DStringAppend(&buffer, contextIclsPtr->nsPtr->fullName, -1);
     varNsPtr = Tcl_FindNamespace(interp, Tcl_DStringValue(&buffer),
         NULL, 0);
@@ -1824,9 +1809,6 @@ Itcl_AddComponentCmd(
     vlookup->accessible = (ivPtr->protection != ITCL_PRIVATE ||
         ivPtr->iclsPtr == contextIclsPtr);
 
-    if (ivPtr->flags & ITCL_COMMON) {
-        type = VAR_TYPE_COMMON;
-    }
     vlookup->varNum = contextIclsPtr->numInstanceVars++;
     /*
      *  Create all possible names for this variable and enter
@@ -1920,7 +1902,6 @@ Itcl_SetComponentCmd(
     Tcl_Obj *const objv[])   /* argument objects */
 {
     FOREACH_HASH_DECLS;
-    ItclObjectInfo *infoPtr;
     ItclClass *iclsPtr;
     ItclObject *contextIoPtr;
     ItclClass *contextIclsPtr;
@@ -1933,7 +1914,6 @@ Itcl_SetComponentCmd(
 
     result = TCL_OK;
     contextIoPtr = NULL;
-    infoPtr = (ItclObjectInfo *)clientData;
     ItclShowArgs(1, "Itcl_SetComponentCmd", objc, objv);
     if (objc < 4) {
         Tcl_WrongNumArgs(interp, 1, objv, 
@@ -2008,10 +1988,8 @@ Itcl_ExtendedClassCmd(
     Tcl_Obj *const objv[])   /* argument objects */
 {
     ItclClass *iclsPtr;
-    ItclObjectInfo *infoPtr;
     int result;
 
-    infoPtr = (ItclObjectInfo *)clientData;
     ItclShowArgs(1, "Itcl_ExtendedClassCmd", objc-1, objv);
     result = ItclClassBaseCmd(clientData, interp, ITCL_ECLASS, objc, objv,
             &iclsPtr);
@@ -2044,10 +2022,8 @@ Itcl_TypeClassCmd(
 {
     Tcl_Obj *objPtr;
     ItclClass *iclsPtr;
-    ItclObjectInfo *infoPtr;
     int result;
 
-    infoPtr = (ItclObjectInfo *)clientData;
     ItclShowArgs(1, "Itcl_TypeClassCmd", objc-1, objv);
     result = ItclClassBaseCmd(clientData, interp, ITCL_TYPE, objc, objv,
             &iclsPtr);
@@ -2092,9 +2068,7 @@ Itcl_ClassHullTypeCmd(
     ItclObjectInfo *infoPtr;
     const char *hullTypeName;
     int correctArg;
-    int result;
 
-    result = TCL_OK;
     infoPtr = (ItclObjectInfo *)clientData;
     iclsPtr = (ItclClass*)Itcl_PeekStack(&infoPtr->clsStack);
     ItclShowArgs(1, "Itcl_ClassHullTypeCmd", objc-1, objv);
