@@ -236,7 +236,6 @@ Itcl_CreateEnsemble(
     const char **nameArgv = NULL;
     int nameArgc;
     Ensemble *parentEnsData;
-    Tcl_DString buffer;
 
     /*
      *  Split the ensemble name into its path components.
@@ -289,12 +288,9 @@ ensCreateFail:
     if (nameArgv) {
         ckfree((char*)nameArgv);
     }
-    Tcl_DStringInit(&buffer);
-    Tcl_DStringAppend(&buffer, "\n    (while creating ensemble \"", -1);
-    Tcl_DStringAppend(&buffer, ensName, -1);
-    Tcl_DStringAppend(&buffer, "\")", -1);
-    Tcl_AddObjErrorInfo(interp, Tcl_DStringValue(&buffer), -1);
-    Tcl_DStringFree(&buffer);
+    Tcl_AppendObjToErrorInfo(interp, Tcl_ObjPrintf(
+            "\n    (while creating ensemble \"%s\")",
+            ensName));
 
     return TCL_ERROR;
 }
@@ -342,7 +338,6 @@ Itcl_AddEnsemblePart(
     int nameArgc;
     Ensemble *ensData;
     EnsemblePart *ensPart;
-    Tcl_DString buffer;
 
     /*
      *  Parse the ensemble name and look for a containing ensemble.
@@ -380,12 +375,9 @@ ensPartFail:
     if (nameArgv) {
         ckfree((char*)nameArgv);
     }
-    Tcl_DStringInit(&buffer);
-    Tcl_DStringAppend(&buffer, "\n    (while adding to ensemble \"", -1);
-    Tcl_DStringAppend(&buffer, ensName, -1);
-    Tcl_DStringAppend(&buffer, "\")", -1);
-    Tcl_AddObjErrorInfo(interp, Tcl_DStringValue(&buffer), -1);
-    Tcl_DStringFree(&buffer);
+    Tcl_AppendObjToErrorInfo(interp, Tcl_ObjPrintf(
+            "\n    (while adding to ensemble \"%s\")",
+            ensName));
 
     return TCL_ERROR;
 }
@@ -824,18 +816,14 @@ CreateEnsemble(
     if (ensData->nsPtr == NULL) {
         Tcl_AppendResult(interp, "error in creating namespace: ",
 	        Tcl_DStringValue(&buffer), NULL);
-        return TCL_ERROR;
+        result = TCL_ERROR;
+        goto finish;
     }
 
     /*
      *  If there is no parent data, then this is a top-level
      *  ensemble.  Create the ensemble by installing its access
      *  command.
-     *
-     *  BE CAREFUL:  Set the string-based proc to the wrapper
-     *    procedure TclInvokeObjectCommand.  Otherwise, the
-     *    ensemble command may fail.  For example, it will fail
-     *    when invoked as a hidden command.
      */
     if (parentEnsData == NULL) {
 	Tcl_Obj *unkObjPtr;
@@ -872,7 +860,7 @@ CreateEnsemble(
         result = TCL_ERROR;
         goto finish;
     }
-    Tcl_DStringInit(&buffer);
+    Tcl_DStringSetLength(&buffer, 0);
     Tcl_DStringAppend(&buffer, infoPtr->ensembleInfo->ensembleNsPtr->fullName, -1);
     Tcl_DStringAppend(&buffer, "::subensembles::", -1);
     sprintf(buf, "%d", parentEnsData->ensembleId);
@@ -1223,7 +1211,7 @@ CreateEnsemblePart(
     if (ensData->numParts >= ensData->maxParts) {
         size = ensData->maxParts*sizeof(EnsemblePart*);
         partList = (EnsemblePart**)ckalloc((unsigned)2*size);
-        memcpy((VOID*)partList, (VOID*)ensData->parts, (size_t)size);
+        memcpy(partList, ensData->parts, (size_t)size);
         ckfree((char*)ensData->parts);
 
         ensData->parts = partList;
@@ -1284,7 +1272,6 @@ static void
 DeleteEnsemblePart(
     ClientData clientData)     /* part being destroyed */
 {
-    Tcl_Command cmdPtr;
     Tcl_Obj *mapDict;
     Tcl_HashEntry *hPtr;
     ItclObjectInfo *infoPtr;
@@ -1299,7 +1286,6 @@ DeleteEnsemblePart(
     if (ensPart == NULL) {
         return;
     }
-    cmdPtr = ensPart->cmdPtr;
     ensData = ensPart->ensemble;
 
     /*
@@ -1332,10 +1318,10 @@ DeleteEnsemblePart(
 	        Tcl_SetEnsembleMappingDict(NULL, ensData2->cmdPtr, mapDict);
 	    }
 	}
-        Tcl_DecrRefCount(ensPart->subEnsemblePtr);
-if (ensPart->newMapDict != NULL) {
-    Tcl_DecrRefCount(ensPart->newMapDict);
-}
+	Tcl_DecrRefCount(ensPart->subEnsemblePtr);
+	if (ensPart->newMapDict != NULL) {
+	    Tcl_DecrRefCount(ensPart->newMapDict);
+	}
     }
     /*
      *  Find this part within its ensemble, and remove it from
@@ -1720,21 +1706,22 @@ Itcl_EnsembleCmd(
     ensName = Tcl_GetString(objv[1]);
 
     if (ensData) {
-        if (FindEnsemblePart(interp, ensData, ensName, &ensPart) != TCL_OK) {
+        if (FindEnsemblePart(ensInfo->master, ensData, ensName, &ensPart) != TCL_OK) {
             ensPart = NULL;
         }
         if (ensPart == NULL) {
-            if (CreateEnsemble(interp, ensData, ensName) != TCL_OK) {
+            if (CreateEnsemble(ensInfo->master, ensData, ensName) != TCL_OK) {
+		Tcl_TransferResult(ensInfo->master, TCL_ERROR, interp);
                 return TCL_ERROR;
             }
-            if (FindEnsemblePart(interp, ensData, ensName, &ensPart)
+            if (FindEnsemblePart(ensInfo->master, ensData, ensName, &ensPart)
                     != TCL_OK) {
                 Tcl_Panic("Itcl_EnsembleCmd: can't create ensemble");
             }
         }
 
         cmd = ensPart->cmdPtr;
-        infoPtr = Tcl_GetAssocData(interp, ITCL_INTERP_DATA, &procPtr);
+        infoPtr = Tcl_GetAssocData(ensInfo->master, ITCL_INTERP_DATA, &procPtr);
         hPtr = Tcl_FindHashEntry(&infoPtr->ensembleInfo->ensembles,
 	        (char *)ensPart->cmdPtr);
         if (hPtr == NULL) {
@@ -1790,12 +1777,12 @@ Itcl_EnsembleCmd(
     ensInfo->ensData = ensData;
 
     if (objc == 3) {
-        status = Tcl_EvalObj(ensInfo->parser, objv[2]);
+        status = Tcl_EvalObjEx(ensInfo->parser, objv[2], 0);
     } else {
         if (objc > 3) {
             objPtr = Tcl_NewListObj(objc-2, objv+2);
             Tcl_IncrRefCount(objPtr);  /* stop Eval trashing it */
-            status = Tcl_EvalObj(ensInfo->parser, objPtr);
+            status = Tcl_EvalObjEx(ensInfo->parser, objPtr, 0);
             Tcl_DecrRefCount(objPtr);  /* we're done with the object */
         }
     }
@@ -1808,6 +1795,7 @@ Itcl_EnsembleCmd(
      */
     if (status == TCL_ERROR) {
 #ifdef NOTDEF
+#else
 	/* no longer needed, no extra interpreter !! */
         const char *errInfo = Tcl_GetVar2(ensInfo->parser, "::errorInfo",
             (char*)NULL, TCL_GLOBAL_ONLY);
@@ -1818,17 +1806,9 @@ Itcl_EnsembleCmd(
 #endif
 
         if (objc == 3) {
-            char msg[128];
-	    Tcl_Obj *options = Tcl_GetReturnOptions(interp, status);
-	    Tcl_Obj *key = Tcl_NewStringObj("-errorline", -1);
-	    Tcl_Obj *stackTrace;
-	    Tcl_IncrRefCount(key);
-	    Tcl_DictObjGet(NULL, options, key, &stackTrace);
-	    Tcl_DecrRefCount(key);
-            sprintf(msg, "\n    (\"ensemble\" body line %s)",
-                Tcl_GetString(stackTrace));
-/*                ensInfo->parser->errorLine); */
-            Tcl_AddObjErrorInfo(interp, msg, -1);
+	    Tcl_AppendObjToErrorInfo(interp, Tcl_ObjPrintf(
+	            "\n    (\"ensemble\" body line %d)",
+		    Tcl_GetErrorLine(ensInfo->parser)));
         }
     }
     Tcl_SetObjResult(interp, Tcl_GetObjResult(ensInfo->parser));
@@ -1862,7 +1842,6 @@ static EnsembleParser*
 GetEnsembleParser(
     Tcl_Interp *interp)     /* interpreter handling the ensemble */
 {
-    Tcl_Namespace *nsPtr;
     EnsembleParser *ensInfo;
 
     /*
@@ -1882,16 +1861,16 @@ GetEnsembleParser(
      */
     ensInfo = (EnsembleParser*)ckalloc(sizeof(EnsembleParser));
     ensInfo->master = interp;
-    ensInfo->parser = interp;
+    ensInfo->parser = Tcl_CreateInterp();
     ensInfo->ensData = NULL;
 
+#ifdef NOTDEF
     /*
      *  Remove all namespaces and all normal commands from the
      *  parser interpreter.
      */
     nsPtr = Tcl_GetGlobalNamespace(ensInfo->parser);
 
-#ifdef NOTDEF
     for (hPtr = Tcl_FirstHashEntry(&nsPtr->childTable, &search);
          hPtr != NULL;
          hPtr = Tcl_FirstHashEntry(&nsPtr->childTable, &search)) {
@@ -1908,6 +1887,8 @@ GetEnsembleParser(
         Tcl_DeleteCommandFromToken(ensInfo->parser, cmd);
     }
 
+#else
+    Tcl_DeleteNamespace(Tcl_GetGlobalNamespace(ensInfo->parser));
 #endif
     /*
      *  Add the allowed commands to the parser interpreter:
@@ -2000,7 +1981,6 @@ Itcl_EnsPartCmd(
 {
     Tcl_Obj *usagePtr;
     Tcl_Proc procPtr;
-    Tcl_Command cmdPtr;
     EnsembleParser *ensInfo = (EnsembleParser*)clientData;
     Ensemble *ensData = (Ensemble*)ensInfo->ensData;
     EnsemblePart *ensPart;
@@ -2029,7 +2009,6 @@ Itcl_EnsPartCmd(
      *  accessed through the ensemble, not through a Tcl command.
      */
     partName = Tcl_GetStringFromObj(objv[1], (int*)NULL);
-    cmdPtr = ensData->cmdPtr;
 
     if (ItclCreateArgList(interp, Tcl_GetString(objv[2]), &argc, &maxArgc,
             &usagePtr, &arglistPtr, NULL, partName) != TCL_OK) {
@@ -2040,8 +2019,9 @@ Itcl_EnsPartCmd(
 	result = TCL_ERROR;
 	goto errorOut;
     }
-    if (Tcl_CreateProc(interp, cmdInfo.namespacePtr, partName, objv[2], objv[3],
+    if (Tcl_CreateProc(ensInfo->master, cmdInfo.namespacePtr, partName, objv[2], objv[3],
             &procPtr) != TCL_OK) {
+	Tcl_TransferResult(ensInfo->master, TCL_ERROR, interp);
 	result = TCL_ERROR;
 	goto errorOut;
     }
@@ -2054,9 +2034,10 @@ Itcl_EnsPartCmd(
      *  if we try to compile the Tcl code for the part.  If
      *  anything goes wrong, clean up before bailing out.
      */
-    result = AddEnsemblePart(interp, ensData, partName, usage,
+    result = AddEnsemblePart(ensInfo->master, ensData, partName, usage,
         Tcl_GetObjInterpProc(), (ClientData)procPtr, _Tcl_ProcDeleteProc,
         ITCL_ENSEMBLE_ENSEMBLE, &ensPart);
+    Tcl_TransferResult(ensInfo->master, result, interp);
 
 errorOut:
     Tcl_DecrRefCount(usagePtr);
